#!/bin/sh
NAS_ARCH=armhf
NAS_MODEL=nas5xx
BSPDEBVER=5.21

BSP_BUILD_ROOT=`pwd`/linux-bsp-${NAS_MODEL}


mkdir -p "${BSP_BUILD_ROOT}"


getproccmdline() {
val=
for x in $(cat /proc/cmdline); do
    case "$x" in
        $1=*)
            val="${x#$1=}"
    esac
done
echo -n $val
}


BOOTPATH=/boot
if [ ! -e ${BOOTPATH}/debroot.sh ]; then
  BOOTPATH=/mnt/partnerkey
fi

if [ ! -e ${BOOTPATH}/debroot.sh ]; then
  ROOTDEV=`getproccmdline root`

  if echo $ROOTDEV | grep '=' > /dev/null ; then
    ROOTDEV=`sudo findfs ${ROOTDEV}`
  fi

  BOOTDEV=`echo $ROOTDEV | tr 2 1`

  if [ -e ${BOOTDEV} ]; then
    sudo mkdir -p ${BOOTPATH}
    sudo mount -o defaults,ro ${BOOTDEV} ${BOOTPATH}
  fi
fi

if [ ! -e ${BOOTPATH}/debroot.sh ]; then
  exit 1
fi


cp -p install-bsp-boot.sh ${BSP_BUILD_ROOT}/


BSPDEBNAM=linux-bsp-${NAS_MODEL}-boot
BSPDEBDIR=${BSP_BUILD_ROOT}/${BSPDEBNAM}_${BSPDEBVER}_${NAS_ARCH}

cd ${BOOTPATH}/

sudo tar czvf ${BSP_BUILD_ROOT}/${BSPDEBNAM}-flat.tar.gz \
  debroot.sh \
  md5sum \
  *_check_file* \
  README \
  salted_md5sum* \
  usb_key_func*

cd - > /dev/null

mkdir -p "${BSPDEBDIR}/boot"

cd "${BSPDEBDIR}/boot"

fakeroot tar xzf ${BSP_BUILD_ROOT}/${BSPDEBNAM}-flat.tar.gz

cd - > /dev/null

fakeroot mkdir -p ${BSPDEBDIR}
fakeroot mkdir    ${BSPDEBDIR}/DEBIAN

#sudo chmod 755 ${BSPDEBDIR}/DEBIAN

echo "/boot/usb_key_func.sh.2" > ${BSPDEBDIR}/DEBIAN/conffiles
#echo "/usr/local/etc/zy-nand-profile" >> ${BSPDEBDIR}/DEBIAN/conffiles

cat <<EOFDRUBSPC | sudo tee ${BSPDEBDIR}/DEBIAN/control > /dev/null
Package: ${BSPDEBNAM}
Architecture: ${NAS_ARCH}
Maintainer: SCP
Depends: i2c-tools, mtd-utils
Priority: optional
Version: ${BSPDEBVER}
Description: ${NAS_MODEL} BSP boot
EOFDRUBSPC

find ${BSPDEBDIR} -type d -exec chmod g-s {} \;

cd "${BSP_BUILD_ROOT}"
fakeroot dpkg-deb --build ${BSPDEBDIR}

cd - > /dev/null

#cp -p ${BSP_BUILD_ROOT}/${BSPDEBNAM}_${BSPDEBVER}_${NAS_ARCH}.deb .


cd ${BSP_BUILD_ROOT}/

zip ${BSP_BUILD_ROOT}/${BSPDEBNAM}-${BSPDEBVER}-${NAS_ARCH}.zip install-bsp-boot.sh linux-bsp-*-boot_*_*.deb linux-bsp-*-init_*_*.deb

cd - > /dev/null

