#!/bin/sh
. ./combine-env.sh

lxsocta=linux-${lxsocka}
lxdebta=linux_${lxdebva}
lxdebtb=linux_${lxdebvb}

lxsocea=.third
#lxsocea=.zyxel
lxdebua=https://www.kernel.org/pub/linux/kernel/v3.x
lxdebea=.mainline
lxsoceb=-third
#lxsoceb=-zyxel

scriptdir=`dirname %0`
cd ${scriptdir} ; scriptdir=`pwd` ; cd - > /dev/null
archivesdir=${scriptdir}/archives

lxdynexclude=false
lxrunforce=true

# functions

keep() {
  for f in `cat $1` ; do
    of=$2/$f
    nf=$3/$f
    od=`dirname $of`
    nd=`dirname $nf`
    if [ -e $of ]; then
      cp -p $of $nd
    fi
  done
}

ignore() {
  for f in `cat $1` ; do
    of=$2/$f
    nf=$2-ignore/$f
    od=`dirname $of`
    nd=`dirname $nf`
    if [ -e $of ]; then
      mkdir -p $nd
      mv $of $nd
    fi
  done
}

restore() {
  for f in `cat $1` ; do
    of=$2/$f
    nf=$2-ignore/$f
    od=`dirname $of`
    nd=`dirname $nf`
    if [ -e $nf ]; then
      mkdir -p $od
      mv $nf $od
    fi
  done

  if [ -e $2-ignore ]; then
    for n in `seq 1 9`; do
      if [ -e $2-ignore ]; then
        find $2-ignore/ -type d -print | xargs -i /bin/rmdir --ignore-fail-on-non-empty {}
      fi
    done
  fi
  if [ -e $2-ignore ]; then
    rmdir --ignore-fail-on-non-empty $2-ignore
  fi
}

lxdiff() {
  #--exclude=powerpc

  if [ "x$4" = "x-wbB" ]; then
    diff -wbBurN \
      --ignore-matching-lines='.*\$Header: .*' \
      --ignore-matching-lines='.*\$Date: .*'     --ignore-matching-lines='.*\$Id\$' \
      --ignore-matching-lines='.*\$Id: .*'       --ignore-matching-lines='.*\$Log: .*' \
      --ignore-matching-lines='.*\$Revision: .*' --ignore-matching-lines='.* \* Revision .*' \
      --ignore-matching-lines='^Revision .*'     --ignore-matching-lines='^log message' \
      --ignore-matching-lines='^ \*.*' \
      --exclude='.gitignore' --exclude='.mailmap' --exclude='*.orig' $1 $2 > $3
  else
    diff -urN --exclude='.gitignore' --exclude='*.orig' $1 $2 > $3
  fi

  if [ "x$4" = "x-wbB" ]; then
    sed -i 's|^diff -wbBurN .* '$1'/|diff -wbBurN '$1'/|g' $3
  else
    sed -i 's|^diff -urN .* '$1'/|diff -urN '$1'/|g' $3
  fi
}

lxdifflist() {
  rm -f $4
  for f in `cat $1` ; do
    #--exclude=powerpc

    if [ "x$5" = "x-wbB" ]; then
      diff -wbBurN \
      --ignore-matching-lines='.*\$Header: .*' \
      --ignore-matching-lines='.*\$Date: .*'     --ignore-matching-lines='.*\$Id\$' \
      --ignore-matching-lines='.*\$Id: .*'       --ignore-matching-lines='.*\$Log: .*' \
      --ignore-matching-lines='.*\$Revision: .*' --ignore-matching-lines='.* \* Revision .*' \
      --ignore-matching-lines='^Revision .*'     --ignore-matching-lines='^log message' \
      --ignore-matching-lines='^ \*.*' \
      --exclude='.gitignore' --exclude='.mailmap' --exclude='*.orig' $2/$f $3/$f >> $4
    else
      diff -urN --exclude='.gitignore' --exclude='*.orig' $2/$f $3/$f >> $4
    fi
  done

  if [ "x$5" = "x-wbB" ]; then
    sed -i 's|^diff -wbBurN .* '$2'/|diff -wbBurN '$2'/|g' $4
  else
    sed -i 's|^diff -urN .* '$2'/|diff -urN '$2'/|g' $4
  fi
}

dynexclude()
{
    lxa=$1
    lxb=$2
    lpa=z-wb-$3
    lpb=$3
    det=$4

    cat ${lpa} | grep -E '^\+\+\+ b/' | cut -f1 | sed 's|^+++ b/||g' > _a.txt
    cat ${lpb} | grep -E '^\+\+\+ b/' | cut -f1 | sed 's|^+++ b/||g' > _b.txt
    diff -ur _a.txt _b.txt | grep -E '^\+' | cut -d + -f2- | grep -v -E '^\+\+' > ${det}

    mv ${lpb} y-${lpb}

    echo "Moving dynamic-exclude files from ${lxa}..."
    ignore ${det} ${lxa}
    echo "Moving dynamic-exclude files from ${lxb}..."
    ignore ${det} ${lxb}
}

dynrestore()
{
    lxa=$1
    lxb=$2
    det=$3

    echo "Restoring dynamic-exclude files from ${lxa}..."
    restore ${det} ${lxa}
    echo "Restoring dynamic-exclude files from ${lxb}..."
    restore ${det} ${lxb}
}

lxdyndifflist() {
  lxl=$1
  lxa=$2
  lxb=$3
  lxp=$4
  lxe=$5

  lxn=`basename ${lxl}`
  lxn=${lxn/combine-/}
  lxn=${lxn/.txt/}

  echo "Comparing ${lxa} nohw to ${lxb} ${lxn}..."
  lxdifflist ${lxl} a b ${lxp}

  if $lxdynexclude ; then
    echo "Comparing ${lxa} nohw to ${lxb} (with -wbB) ${lxn}..."
    lxdifflist ${lxl} a b z-wb-${lxp} -wbB

    dynexclude ${lxa} ${lxb} ${lxp} ${lxe}

    echo "Comparing ${lxa} nohw to ${lxb} ${lxn} again..."
    lxdifflist ${lxl} a b ${lxp}

    dynrestore ${lxa} ${lxb} ${lxe}
  fi

  p=${lxa}-base
  echo "Patching ${lxa}..."
  cd ${lxa}/
  patch -p1 < ../${lxp} 2>&1 1>../${p}.patch.log
  r=$?
  if [ $r != 0 ]; then
    cat ../${p}.patch.log | grep 'saving rejects to file' | cut -d - -f 2- | cut -d ' ' -f 6 | sed s/'.rej'/''/g
    echo ...failed
    exit $r
  fi
  cd ..
}

extradiff() {
  rm -f c
  rm -f d
  ln -s $2 c
  ln -s $3 d
  for f in `cat $1` ; do
    # "x${f:0:7}" != "xfs/ext4"
    # "x${f}" != "xfs/ext4/file.c"
    # "x${f}" != "xdrivers/md/md.c"
    if [ "x$4" = "x--force" ]; then
      diff -urN --exclude='*.orig' c/$f d/$f >> linux-${lxsocka}-to-${lxdeblb}-extra.patch
    elif [ "x${f}" != "xarch/arm/include/asm/pgtable.h" -a "x${f}" != "xdrivers/char/random.c" -a "x${f}" != "xinclude/linux/random.h" \
         -a "x${f}" != "xdrivers/usb/core/hub.c" -a "x${f}" != "xdrivers/usb/host/xhci-hub.c" -a "x${f}" != "xdrivers/usb/host/xhci-mem.c" -a "x${f}" != "xinit/main.c" -a "x${f}" != "xkernel/irq/handle.c" \
         -a "x${f}" != "xinclude/linux/if_vlan.h" -a "x${f}" != "xinclude/net/rtnetlink.h" -a "x${f}" != "xnet/8021q/vlan_core.c" -a "x${f}" != "xnet/core/dev.c" -a "x${f}" != "xnet/core/rtnetlink.c" \
         -a "x${f}" != "xblock/blk-settings.c" -a "x${f:0:11}" != "xdrivers/ata" \
         -a "x${f}" != "xdrivers/md/dm-crypt.c" -a "x${f}" != "xdrivers/md/raid5.c" -a "x${f}" != "xnet/key/af_key.c" \
         -a "x${f}" != "xkernel/softirq.c" -a "x${f:0:15}" != "xdrivers/mtd/ubi" \
         -a "x${f}" != "xdrivers/usb/host/xhci.c" -a "x${f}" != "xdrivers/usb/host/xhci.h" \
         -a "x${f}" != "xdrivers/usb/host/xhci-pci.c" -a "x${f}" != "xdrivers/usb/host/xhci-ring.c" ]; then
      diff -urN --exclude='*.orig' c/$f d/$f >> linux-${lxsocka}-to-${lxdeblb}-extra.patch
    fi
  done
  rm -f c
  rm -f d
}

# main

mkdir -p combine

mkdir -p ${archivesdir}
cd ${archivesdir}

if [ ! -e linux_${lxdebka}${lxdebea}.tar.xz ]; then
  echo "Downloading debian linux_${lxdebka}..."
  wget -N "${lxdebua}/linux_${lxdebka}${lxdebea}.tar.xz" || true
  if [ ! -e linux_${lxdebka}${lxdebea}.tar.xz ]; then
    wget "${lxdebua}/linux-${lxdebka}.tar.xz" -O linux_${lxdebka}${lxdebea}.tar.xz
  fi
fi

if [ ! -e linux_${lxdebkb}.orig.tar.xz -o ! -e linux_${lxdebvb}.debian.tar.xz ]; then
  echo "Downloading debian linux_${lxdebkb}..."
  wget -N "${lxdebub}/linux_${lxdebkb}.orig.tar.xz" || true
  if [ ! -e linux_${lxdebkb}.orig.tar.xz ]; then
    wget "${lxdebub}/linux-${lxdebkb}.tar.xz" -O linux_${lxdebkb}.orig.tar.xz
  fi
  wget -N "${lxdebub}/linux_${lxdebvb}.debian.tar.xz"
  wget -N "${lxdebub}/linux_${lxdebvb}.dsc"
fi

if [ ! -e linux-${lxsocka}.tar.xz ]; then
  echo "Downloading linux-${lxsocka}..."
  wget -N "${lxsocua}/linux-${lxsocka}.tar.xz"
fi

cd - > /dev/null


cd combine

if [ -e linux-${lxsocka}${lxsocea} ]; then
  restore ../lists/combine-exclude-all.txt linux-${lxsocka}${lxsocea}
  restore ../lists/combine-exclude-c2k-extra.txt linux-${lxsocka}${lxsocea}

  if [ -e linux-${lxsocka}${lxsocea}/include/qnap ]; then
    restore ../lists/combine-exclude-archs.txt linux-${lxsocka}${lxsocea}
    restore ../lists/combine-exclude-target.txt linux-${lxsocka}${lxsocea}
    restore ../lists/combine-exclude-qnap.txt linux-${lxsocka}${lxsocea}

  elif [ -e linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
    restore ../lists/combine-exclude-google.txt linux-${lxsocka}${lxsocea}
  fi
else
  echo "Extracting linux-${lxsocka}${lxsocea}..."
  tar xJf ${archivesdir}/linux-${lxsocka}.tar.xz 
  mv linux-${lxsocka} linux-${lxsocka}${lxsocea}
fi

if [ ! -e linux-${lxdebka}${lxdebea} ]; then
  echo "Extracting linux_${lxdebka}${lxdebea}..."
  tar xJf ${archivesdir}/linux_${lxdebka}${lxdebea}.tar.xz
  mv linux-${lxdebka} linux-${lxdebka}${lxdebea}
fi

if [ ! -e linux-${lxdebkb}.orig ]; then
  echo "Extracting linux_${lxdebkb}.orig..."
  tar xJf ${archivesdir}/linux_${lxdebkb}.orig.tar.xz
  mv linux-${lxdebkb} linux-${lxdebkb}.orig
fi

cd ..


if false; then
#if [ ! -e combine/linux-${lxdebka}-generic ]; then
  cd combine
  echo "Extracting linux_${lxdebka}${lxdebea} to generic..."
  tar xJf ${archivesdir}/linux_${lxdebka}${lxdebea}.tar.xz 
  mv linux-${lxdebka} linux-${lxdebka}-generic
  cp -ap linux-${lxsocka}${lxsocea}/patches linux-${lxdebka}-generic/

  p=linux-${lxdebka}-generic
  echo "Patching linux_${lxdebka}${lxdebea}..."
  cd linux-${lxdebka}-generic
  sed -i s/'^platform.*'/''/g patches/series 
  quilt push -a 2>&1 1>../${p}.patch.log
  r=$?
  cd ..

  if [ $r != 0 ]; then
    cat ${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
    echo Failed $r
    exit $r
  fi

  cd ..
fi

if [ ! -e combine/linux-${lxdebka}-platform ]; then
  cd combine
  echo "Extracting linux_${lxdebka}${lxdebea} to platform..."
  tar xJf ${archivesdir}/linux_${lxdebka}${lxdebea}.tar.xz 
  mv linux-${lxdebka} linux-${lxdebka}-platform

  if [ -e linux-${lxsocka}${lxsocea}/patches ]; then
    cp -ap linux-${lxsocka}${lxsocea}/patches linux-${lxdebka}-platform/
  fi

  if [ -e linux-${lxsocka}${lxsocea}/patches/series ]; then
    p=linux-${lxdebka}-platform
    echo "Patching linux_${lxdebka}${lxdebea}..."
    cd linux-${lxdebka}-platform
    quilt push -a 2>&1 1>../${p}.patch.log
    r=$?
    cd ..

    if [ $r != 0 ]; then
      cat ${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
      echo Failed $r
      exit $r
    fi
  fi

  cd ..
fi

if [ -e combine/linux-${lxsocka}${lxsocea}/drivers/hwmon/wd-fan.c ]; then
  lxrunforce=false
elif [ -e combine/linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
  lxrunforce=false
elif [ -e combine/linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/synology-gpio.c ]; then
  lxdynexclude=true
  lxrunforce=false
elif [ -e combine/linux-${lxsocka}${lxsocea}/include/qnap ]; then
  lxdynexclude=true
  lxrunforce=false
elif grep -q 'drivers/usb/host/xhci\.h' lists/combine-keep.txt ; then
  lxrunforce=false
fi

if [ ! -e combine/054-mspd-kernel_linux_3_2_extra.patch ]; then
  cd combine

  cd linux-${lxdebka}-platform

  for f in `cat patches/series | grep -E '^generic/' | grep -v -E \
'201-extra_optimization|'\
'920-unable_to_open_console|'\
'921-use_preinit_as_init|'\
'950-vm_exports' | sort -r` ; do
    echo $f ...
    patch -p1 -R < patches/$f || exit 1
    r=$?
    if [ $r != 0 ]; then
      echo ...failed
      exit $r
    fi

  done

  cd ..

  #echo "r1"
  #read f

  cd linux-${lxsocka}${lxsocea}

  for f in `cat patches/series | grep -E '^generic/' | grep -v -E \
'201-extra_optimization|'\
'920-unable_to_open_console|'\
'921-use_preinit_as_init|'\
'950-vm_exports' | sort -r` ; do
    echo $f ...
    patch -p1 -R < patches/$f || exit 1
    r=$?
    if [ $r != 0 ]; then
      echo ...failed
      exit $r
    fi
  done

  cd ..

  #echo "r2"
  #read f

  cd ..

  ignore lists/combine-exclude-all.txt combine/linux-${lxdebka}-platform
  ignore lists/combine-exclude-all.txt combine/linux-${lxdebka}-generic
  ignore lists/combine-exclude-all.txt combine/linux-${lxsocka}${lxsocea}
  ignore lists/combine-exclude-c2k-extra.txt combine/linux-${lxsocka}${lxsocea}

  if [ -e combine/linux-${lxsocka}${lxsocea}/include/qnap ]; then
    ignore lists/combine-exclude-archs.txt combine/linux-${lxdebka}-platform
    ignore lists/combine-exclude-archs.txt combine/linux-${lxdebka}-generic
    ignore lists/combine-exclude-archs.txt combine/linux-${lxsocka}${lxsocea}
    ignore lists/combine-exclude-target.txt combine/linux-${lxdebka}-platform
    ignore lists/combine-exclude-target.txt combine/linux-${lxdebka}-generic
    ignore lists/combine-exclude-target.txt combine/linux-${lxsocka}${lxsocea}
    ignore lists/combine-exclude-qnap.txt combine/linux-${lxdebka}-platform
    ignore lists/combine-exclude-qnap.txt combine/linux-${lxdebka}-generic
    ignore lists/combine-exclude-qnap.txt combine/linux-${lxsocka}${lxsocea}

  elif [ -e combine/linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
    ignore lists/combine-exclude-google.txt combine/linux-${lxdebka}-platform
    ignore lists/combine-exclude-google.txt combine/linux-${lxdebka}-generic
    ignore lists/combine-exclude-google.txt combine/linux-${lxsocka}${lxsocea}
  fi

  cd combine

  echo "Comparing linux-${lxdebka}-platform to linux-${lxsocka}${lxsocea}..."
  extradiff ../lists/combine-exclude-c2k-nohw.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea}

  if [ ! -e linux-${lxsocka}${lxsocea}/include/qnap ]; then
    extradiff ../lists/combine-exclude-c2k-nohw-fs.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea}
  fi

  mv linux-${lxsocka}-to-${lxdeblb}-extra.patch 054-mspd-kernel_linux_3_2_nohw.patch
  p=linux-${lxdebka}-platform-nohw
  echo "Patching linux-${lxdebka}-platform..."
  cd linux-${lxdebka}-platform/
  patch -p1 < ../054-mspd-kernel_linux_3_2_nohw.patch 2>&1 1>../${p}.patch.log
  r=$?
  if [ $r != 0 ]; then
    cat ../${p}.patch.log | grep 'saving rejects to file' | cut -d - -f 2- | cut -d ' ' -f 6 | sed s/'.rej'/''/g
    echo ...failed
    exit $r
  fi
  cd ..

  rm -f a
  rm -f b
  ln -sf linux-${lxdebka}-platform a
  ln -sf linux-${lxsocka}${lxsocea} b

  if [ -e linux-${lxsocka}${lxsocea}/arch/arm/mach-alpine ]; then
    lxdyndifflist ../lists/combine-base-al.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_base-al.patch dynamic-exclude-base-al.txt
  fi
  if [ -e linux-${lxsocka}${lxsocea}/arch/arm/mach-armadaxp ]; then
    lxdyndifflist ../lists/combine-base-mv.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_base-mv.patch dynamic-exclude-base-mv.txt
  fi

  lxdyndifflist ../lists/combine-arme-c2k.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_arme.patch dynamic-exclude-base.txt
  #echo "d1"
  #read f;

  # memf based on:
  # [PATCH 1/3] Rename ARM assembler push/pull macros
  # https://raw.githubusercontent.com/hglm/patches/master/arm-mem-funcs/0001-Rename-ARM-assembler-push-pull-macros.patch
  # [PATCH 2/3] Optimize copy_page for modern ARM platforms
  # https://raw.githubusercontent.com/hglm/patches/master/arm-mem-funcs/0002-Optimize-copy_page-for-modern-ARM-platforms.patch
  # [PATCH 3/3] Optimize ARM memset and memzero functions
  # https://raw.githubusercontent.com/hglm/patches/master/arm-mem-funcs/0003-Optimize-ARM-memset-and-memzero-functions.patch

  lxdyndifflist ../lists/combine-memf-c2k.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_memf.patch dynamic-exclude-base.txt
  #echo "d2"
  #read f;

  # neon based on:
  # [PATCH] ARM: move VFP init to an earlier boot stage
  # https://github.com/libos-nuse/net-next-nuse/commit/0773d73d818702191dd568e1e20bcafc5c64661a.patch
  # [PATCH] ARM: be strict about FP exceptions in kernel mode
  # https://github.com/libos-nuse/net-next-nuse/commit/ab3da15643469ab2d206dee3d9cfa4194ba77f25.patch
  # [PATCH] ARM: add support for kernel mode NEON
  # https://github.com/libos-nuse/net-next-nuse/commit/73c132c15da504789b924871e2491479a18e4f6a.patch
  # [PATCH] ARM: crypto: add NEON accelerated XOR implementation
  # https://github.com/libos-nuse/net-next-nuse/commit/01956597cbc46df072f20f90a40eebe356200c38.patch
  # [PATCH] lib/raid6: add ARM-NEON accelerated syndrome calculation
  # https://github.com/libos-nuse/net-next-nuse/commit/7d11965ddb9b9b1e0a5d13c58345ada1ccbc663b.patch

  lxdyndifflist ../lists/combine-neon-c2k.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_neon.patch dynamic-exclude-base.txt
  #echo "d3"
  #read f;

  lxdyndifflist ../lists/combine-splice-c2k.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_splice.patch dynamic-exclude-base.txt
  #echo "d4"
  #read f;

  lxdyndifflist ../lists/combine-base-c2k.txt linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_base.patch dynamic-exclude-base.txt
  #echo "d5"
  #read f;

  echo "Comparing linux-${lxdebka}-platform nohw base to linux-${lxsocka}${lxsocea}..."
  lxdiff a b 054-mspd-kernel_linux_3_2_extra.patch

  if $lxdynexclude ; then
    echo "Comparing linux-${lxdebka}-platform nohw to linux-${lxsocka}${lxsocea} (with -wbB)..."
    lxdiff a b z-wb-054-mspd-kernel_linux_3_2_extra.patch -wbB

    dynexclude linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} 054-mspd-kernel_linux_3_2_extra.patch dynamic-exclude-extra.txt

    echo "Comparing linux-${lxdebka}-platform nohw to linux-${lxsocka}${lxsocea} again..."
    lxdiff a b 054-mspd-kernel_linux_3_2_extra.patch

    dynrestore linux-${lxdebka}-platform linux-${lxsocka}${lxsocea} dynamic-exclude-extra.txt
  fi

  cd ..

  restore lists/combine-exclude-all.txt combine/linux-${lxdebka}-platform
  restore lists/combine-exclude-all.txt combine/linux-${lxsocka}${lxsocea}
  restore lists/combine-exclude-c2k-extra.txt combine/linux-${lxsocka}${lxsocea}

  if [ -e combine/linux-${lxsocka}${lxsocea}/include/qnap ]; then
    restore lists/combine-exclude-archs.txt combine/linux-${lxdebka}-platform
    restore lists/combine-exclude-archs.txt combine/linux-${lxsocka}${lxsocea}
    restore lists/combine-exclude-target.txt combine/linux-${lxdebka}-platform
    restore lists/combine-exclude-target.txt combine/linux-${lxsocka}${lxsocea}
    restore lists/combine-exclude-qnap.txt combine/linux-${lxdebka}-platform
    restore lists/combine-exclude-qnap.txt combine/linux-${lxsocka}${lxsocea}

  elif [ -e combine/linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
    restore lists/combine-exclude-google.txt combine/linux-${lxdebka}-platform
    restore lists/combine-exclude-google.txt combine/linux-${lxsocka}${lxsocea}
  fi
fi


if false; then
#if [ ! -e combine/linux-${lxdebkb}-generic ]; then
  cd combine

  echo "Extracting linux_${lxdebkb}.orig to generic..."
  tar xJf ${archivesdir}/linux_${lxdebkb}.orig.tar.xz 
  mv linux-${lxdebkb} linux-${lxdebkb}-generic
  cp -ap linux-${lxsocka}${lxsocea}/patches linux-${lxdebkb}-generic/

  p=linux-${lxdebkb}-generic
  echo "Patching linux_${lxdebkb}.orig..."
  keep ../lists/combine-keep.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-generic
  cd linux-${lxdebkb}-generic
  sed -i s/'^platform.*'/''/g patches/series 
  quilt push -a 2>&1 1>../${p}.patch.log
  r=$?
  cd ..

  if [ $r != 0 ]; then
    cat ${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
    echo Failed $r
    exit $r
  fi

  cd ..
fi

if [ ! -e combine/linux-${lxdebkb}-platform ]; then
  cd combine

  echo "Extracting linux_${lxdebkb}.orig to platform..."
  tar xJf ${archivesdir}/linux_${lxdebkb}.orig.tar.xz 
  mv linux-${lxdebkb} linux-${lxdebkb}-platform

  keep ../lists/combine-keep.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-platform
  if [ -e linux-${lxsocka}${lxsocea}/drivers/hwmon/wd-fan.c ]; then
    keep ../lists/combine-keep-wd.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-platform
  elif [ -e linux-${lxsocka}${lxsocea}/include/qnap ]; then
    keep ../lists/combine-keep-qnap.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-platform
  elif [ -e linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
    keep ../lists/combine-keep-google.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-platform
  else
    keep ../lists/combine-keep-c2k.txt linux-${lxdebka}${lxdebea} linux-${lxdebkb}-platform
  fi

  cd linux-${lxdebkb}-platform

  # for 3.2.88 and newer
  sed -i 's|unsigned int border|unsigned short border|g' fs/ext4/mballoc.c

  cd ..

  rm -f a
  rm -f b
  ln -sf linux-${lxdebkb}.orig a
  ln -sf linux-${lxdebkb}-platform b
  echo "Comparing linux-${lxdebkb}.orig linux-${lxdebkb}-platform..."
  lxdiff a b 000-mspd-kernel_linux_${lxdebkb}_revert.patch

  if [ -e linux-${lxsocka}${lxsocea}/patches ]; then
    cp -ap linux-${lxsocka}${lxsocea}/patches linux-${lxdebkb}-platform/
  fi

  if [ -e linux-${lxsocka}${lxsocea}/patches/series ]; then
    p=linux-${lxdebkb}-platform
    echo "Patching linux_${lxdebkb}.orig..."
    cd linux-${lxdebkb}-platform

    mv patches/series patches/series-soc

    echo "generic/201-extra_optimization.patch" >> patches/series
    echo "generic/920-unable_to_open_console.patch" >> patches/series
    echo "generic/921-use_preinit_as_init.patch" >> patches/series
    echo "generic/950-vm_exports.patch" >> patches/series

    cat patches/series-soc | grep -v -E '^generic/' >> patches/series
    #echo "3"
    #read f;

    quilt push -a 2>&1 1>../${p}.patch.log
    r=$?
    cd ..

    if [ $r != 0 ]; then
      cat ${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
      echo Failed $r
      exit $r
    fi
  fi

  if $lxrunforce ; then
    echo "Comparing linux-${lxdebkb}-platform to linux-${lxdebkb}.orig..."

    extradiff ../lists/combine-force-c2k-extra.txt linux-${lxdebkb}-platform linux-${lxdebkb}.orig --force

    mv linux-${lxsocka}-to-${lxdeblb}-extra.patch 084-mspd-kernel_linux_3_2_force.patch
    p=linux-${lxdebkb}-platform-force
    echo "Patching linux-${lxdebkb}-platform..."
    cd linux-${lxdebkb}-platform/
    patch -p1 < ../084-mspd-kernel_linux_3_2_force.patch 2>&1 1>../${p}.patch.log
    r=$?
    if [ $r != 0 ]; then
      cat ../${p}.patch.log | grep 'saving rejects to file' | cut -d - -f 2- | cut -d ' ' -f 6 | sed s/'.rej'/''/g
      echo ...failed
      exit $r
    fi
    cd ..
  fi

  cd ..
fi

if false; then
#if [ -e linux-${lxdebkb}-platform ]; then
  ignore lists/combine-exclude-all.txt combine/linux-${lxdebkb}-platform
  ignore lists/combine-exclude-all.txt combine/linux-${lxdebkb}-generic
  ignore lists/combine-exclude-all.txt combine/linux-${lxsocka}${lxsocea}
  ignore lists/combine-exclude-c2k-extra.txt combine/linux-${lxsocka}${lxsocea}

  cd combine
  rm -f a
  rm -f b
  ln -sf linux-${lxdebkb}-platform a
  ln -sf linux-${lxsocka}${lxsocea} b
  echo "Comparing linux-${lxdebkb}-platform to linux-${lxsocka}${lxsocea}..."
  lxdiff a b 054-mspd-kernel_linux_3_2_extra.patch
  cd ..
fi

if [ ! -e combine/linux-${lxdebkb}${lxsoceb} ]; then
  cd combine

  echo "Extracting linux_${lxdebkb}.orig to ${lxsoceb}..."
  tar xJf ${archivesdir}/linux_${lxdebkb}.orig.tar.xz
  mv linux-${lxdebkb} linux-${lxdebkb}${lxsoceb}

  p=linux-${lxdebkb}${lxsoceb}
  echo "Patching linux_${lxdebkb}.orig..."
  cp -ap linux-${lxsocka}${lxsocea}/drivers/epn1802-ocf linux-${lxdebkb}${lxsoceb}/drivers/
  cp -ap linux-${lxsocka}${lxsocea}/drivers/pfe         linux-${lxdebkb}${lxsoceb}/drivers/
  cp -ap linux-${lxsocka}${lxsocea}/drivers/pfe_ctrl    linux-${lxdebkb}${lxsoceb}/drivers/
  cp -ap linux-${lxsocka}${lxsocea}/firmware            linux-${lxdebkb}${lxsoceb}/
  cp -ap linux-${lxsocka}${lxsocea}/patches             linux-${lxdebkb}${lxsoceb}/
  cd linux-${lxdebkb}${lxsoceb}

  # for 3.2.88 and newer
  #sed -i 's|unsigned int border|unsigned short border|g' fs/ext4/mballoc.c

  mkdir -p patches/platform
  cp -p ../*-mspd-kernel_linux_*.patch patches/platform/
  mv patches/series patches/series-soc
  echo "platform/000-mspd-kernel_linux_${lxdebkb}_revert.patch" >> patches/series

  echo "generic/201-extra_optimization.patch" >> patches/series
  echo "generic/920-unable_to_open_console.patch" >> patches/series
  echo "generic/921-use_preinit_as_init.patch" >> patches/series
  echo "generic/950-vm_exports.patch" >> patches/series

  cat patches/series-soc | grep -v -E '^generic/' >> patches/series
  echo "platform/054-mspd-kernel_linux_3_2_base.patch" >> patches/series

  if [ -e patches/platform/054-mspd-kernel_linux_3_2_arme.patch ]; then
    echo "platform/054-mspd-kernel_linux_3_2_arme.patch" >> patches/series
  fi

  if [ -e patches/platform/054-mspd-kernel_linux_3_2_memf.patch ]; then
    echo "platform/054-mspd-kernel_linux_3_2_memf.patch" >> patches/series
  fi

  if [ -e patches/platform/054-mspd-kernel_linux_3_2_neon.patch ]; then
    echo "platform/054-mspd-kernel_linux_3_2_neon.patch" >> patches/series
  fi

  if [ -e patches/platform/054-mspd-kernel_linux_3_2_splice.patch ]; then
    echo "platform/054-mspd-kernel_linux_3_2_splice.patch" >> patches/series
  fi

  echo "platform/054-mspd-kernel_linux_3_2_extra.patch" >> patches/series
  if [ -e patches/platform/084-mspd-kernel_linux_3_2_force.patch ]; then
    echo "platform/084-mspd-kernel_linux_3_2_force.patch" >> patches/series
  fi

  quilt push -a 2>&1 1>../${p}.patch.log
  r=$?
  cd ..

  if [ $r != 0 ]; then
    cat ${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
    echo Failed $r
    exit $r
  fi

  cd ..
fi

if [ ! -e combine/linux-${lxdebkb}${lxsoceb}/debian ]; then
  cd combine

  cd linux-${lxdebkb}${lxsoceb}/

  tar xJf ${archivesdir}/${lxdebtb}.debian.tar.xz

  p=linux-${lxdebkb}${lxsoceb}-debian
  echo "Patching linux-${lxdebkb}${lxsoceb}..."
  mv patches zy-patches
  mv debian/patches .
  mv zy-patches/generic patches/
  mv zy-patches/platform patches/
  mv patches/series patches/series-deb
  #cp -p zy-patches/series patches/
  # grep -v -E '^generic/'
  cat zy-patches/series > patches/series
  #echo "4"
  #read f;

  # ...

  cat patches/series-deb >> patches/series

  #cat zy-patches/series >> patches/series
  #echo "5"
  #read f;

  if [ ! -e .gitignore ]; then
    sed -i 's|debian/gitignore.patch||g' patches/series
  fi

  #sed -i 's|features/all/debugfs-add-mode-uid-and-gid-options.patch||g' patches/series
  #sed -i 's|debian/debugfs-set-default-mode-to-700.patch||g' patches/series
  #sed -i 's|debian/bh-avoid-abi-change-in-3.2.71.patch||g' patches/series

  #sed -i 's|debian/libata-avoid-abi-change-in-3.2.59.patch||g' patches/series
  #sed -i 's|debian/revert-libata-align-ata_device-s-id-on-a-cacheline.patch||g' patches/series

  #sed -i 's|debian/libata-avoid-abi-change-in-3.2.62.patch||g' patches/series
  ##sed -i 's|debian/revert-libata-ignore-spurious-phy-event-on-lpm-polic.patch||g' patches/series

  ##sed -i 's|debian/fs-fix-abi-change-for-aufs-f_setfl-fix.patch||g' patches/series

  if [ -e ../linux-${lxsocka}${lxsocea}/drivers/hwmon/wd-fan.c ]; then
    sed -i 's|debian/dm-avoid-ABI-change-in-3.2.41.patch||g' patches/series
  fi

  if [ -e ../linux-${lxsocka}${lxsocea}/arch/arm/mach-comcerto/board-optimus.c ]; then
    sed -i 's|debian/sysrq-mask.patch||g' patches/series

    sed -i 's|debian/rtnetlink-avoid-ABI-change-in-3.2.34.patch||g' patches/series

    sed -i 's|debian/dm-avoid-ABI-change-in-3.2.41.patch||g' patches/series
  fi

  if [ -e ../linux-${lxsocka}${lxsocea}/include/qnap ]; then
    sed -i 's|features/arm/rtc-s35390a-wakealarm.patch||g' patches/series
    sed -i 's|bugfix/all/dm-Deal-with-merge_bvec_fn-in-component-devices-bett.patch||g' patches/series

    sed -i 's|debian/rtnetlink-avoid-ABI-change-in-3.2.34.patch||g' patches/series

    cp -p ../../patches/qnap-dm-avoid-ABI-change-in-3.2.41.patch patches/debian/
    sed -i 's|debian/dm-avoid-ABI-change-in-3.2.41.patch|debian/qnap-dm-avoid-ABI-change-in-3.2.41.patch|g' patches/series

    sed -i 's|debian/scsi-avoid-abi-change-in-3.2.61.patch||g' patches/series
    sed -i 's|debian/scsi-avoid-abi-change-in-3.2.62.patch||g' patches/series

    sed -i 's|debian/fs-fix-abi-change-in-3.2.80.patch||g' patches/series
    sed -i 's|debian/fs-move-procfs-ecryptfs-stacking-check-into-ecryptfs.patch||g' patches/series
    sed -i 's|debian/fs-fix-abi-change-in-3.2.84.patch||g' patches/series
  fi

  if grep 'struct resource' include/linux/io.h > /dev/null ; then
    cp -p ../../patches/linux_io_h_add_ioport_h.patch patches/platform/
    echo "platform/linux_io_h_add_ioport_h.patch" >> patches/series
  fi

  if grep 'count_zero_bits' fs/jffs2/scan.c > /dev/null ; then
    cp -p ../../patches/linux-3.2_jffs2-scan-num_zeros.patch patches/platform/
    echo "platform/linux-3.2_jffs2-scan-num_zeros.patch" >> patches/series
  fi

  if grep 'static int set_led_config' arch/arm/mach-comcerto/gpio.c > /dev/null ; then
    cp -p ../../patches/zyxel-nas5xx-gpio-led-fixes.patch patches/platform/
    echo "platform/zyxel-nas5xx-gpio-led-fixes.patch" >> patches/series
  fi

  if grep 'HDD_SET_CTL_IOC_NUM' arch/arm/mach-comcerto/gpio.c > /dev/null ; then
    cp -p ../../patches/zyxel-nas5xx-gpio-mcu-fixes.patch patches/platform/
    echo "platform/zyxel-nas5xx-gpio-mcu-fixes.patch" >> patches/series
  fi

  if grep 'static void __init mac_addr_setup' arch/arm/mach-comcerto/comcerto-2000.c > /dev/null ; then
    cp -p ../../patches/mspd-comcerto-2000-mac_addr_setup.patch patches/platform/
    echo "platform/mspd-comcerto-2000-mac_addr_setup.patch" >> patches/series
  fi

  if grep 'copy_to_user(ppos,&pos,sizeof(loff_t));' fs/splice.c > /dev/null ; then
    if [ -e include/qnap ]; then
      # todo
      true
    else
      cp -p ../../patches/mspd-fs-splice-copy_to_user.patch patches/platform/
      echo "platform/mspd-fs-splice-copy_to_user.patch" >> patches/series
    fi
  fi

  if grep 'void __init c2k_init_gpio' drivers/gpio/gpio-c2k.c > /dev/null ; then
    cp -p ../../patches/mspd-gpio-c2k-init.patch patches/platform/
    echo "platform/mspd-gpio-c2k-init.patch" >> patches/series
  fi

  if grep 'CONFIG_4BAY' drivers/scsi/sd.c > /dev/null ; then
    cp -p ../../patches/zyxel-nas5xx-drive_bays_setup.patch patches/platform/
    echo "platform/zyxel-nas5xx-drive_bays_setup.patch" >> patches/series
  fi

  #cp -p ../../libata-core-sata_lpm_ignore_phy_events.patch patches/platform/
  #cp -p ../../libahci-port_map.patch patches/platform/

  #echo "platform/libata-core-sata_lpm_ignore_phy_events.patch" >> patches/series
  #echo "platform/libahci-port_map.patch" >> patches/series

  if ! grep original_dev_queue_xmit net/core/dev.c > /dev/null ; then
    cp -p ../../patches/mspd-netdevice.patch patches/platform/
    echo "platform/mspd-netdevice.patch" >> patches/series
  fi

  if ! grep IPS_UNCHANGEABLE_MASK include/linux/netfilter/nf_conntrack_common.h > /dev/null ; then
    cp -p ../../patches/mspd-nf_conntrack_common.patch patches/platform/
    echo "platform/mspd-nf_conntrack_common.patch" >> patches/series
  fi

  if ! grep CONFIG_ARCH_COMCERTO include/linux/skbuff.h > /dev/null ; then
    cp -p ../../patches/084-mspd-kernel_linux_3_2_skbuff.patch patches/platform/
    echo "platform/084-mspd-kernel_linux_3_2_skbuff.patch" >> patches/series
  fi

  if ! grep 'setattr_prepare' include/linux/fs.h > /dev/null ; then
    if [ -e include/qnap ]; then
      cp -p ../../patches/qnap-fs_h-fix.patch patches/platform/
      echo "platform/qnap-fs_h-fix.patch" >> patches/series
    fi
  fi

  if ! grep 'sector_t split_io' include/linux/device-mapper.h > /dev/null ; then
      cp -p ../../patches/qnap-device-mapper_h-fix.patch patches/platform/
      echo "platform/qnap-device-mapper_h-fix.patch" >> patches/series
  fi

  if ! grep 'CIFS_MAX_DOMAINNAME_LEN' fs/cifs/cifsglob.h > /dev/null ; then
      cp -p ../../patches/qnap-cifsglob_h-fix.patch patches/platform/
      echo "platform/qnap-cifsglob_h-fix.patch" >> patches/series
  fi

  if ! grep 'CONFIG_MACH_QNAPTS' fs/cifs/cifsglob.h > /dev/null ; then
    if [ -e include/qnap ]; then
      cp -p ../../patches/qnap-cifsglob_h-qnapts-fix.patch patches/platform/
      echo "platform/qnap-cifsglob_h-qnapts-fix.patch" >> patches/series
    fi
  fi

  if ! grep 'CONFIG_MACH_QNAPTS' fs/ext4/ext4.h > /dev/null ; then
    if [ -e include/qnap ]; then
      cp -p ../../patches/qnap-ext4_h-fix.patch patches/platform/
      echo "platform/qnap-ext4_h-fix.patch" >> patches/series
    fi
  fi

  if ! grep 'CONFIG_MACH_QNAPTS' net/socket.c > /dev/null ; then
    if [ -e include/qnap ]; then
      cp -p ../../patches/qnap-socket_c-fix.patch patches/platform/
      echo "platform/qnap-socket_c-fix.patch" >> patches/series
    fi
  fi

  #if grep 'calcit(skb);' net/core/rtnetlink.c > /dev/null ; then
  #    cp -p ../../patches/qnap-rtnetlink-fix.patch patches/platform/
  #    echo "platform/qnap-rtnetlink-fix.patch" >> patches/series
  #fi

  if grep 'rtnl_calcit_func)(struct sk_buff \*, struct nlmsghdr' include/net/rtnetlink.h > /dev/null ; then
    if [ -e include/qnap -o -e arch/arm/mach-comcerto/board-optimus.c ]; then
      cp -p ../../patches/qnap-rtnetlink_h-fix.patch patches/platform/
      echo "platform/qnap-rtnetlink_h-fix.patch" >> patches/series
    fi
  fi

  if ! grep 'NO_REPORT_LUNS' include/linux/usb_usual.h > /dev/null ; then
      cp -p ../../patches/qnap-usb_usual_h-fix.patch patches/platform/
      echo "platform/qnap-usb_usual_h-fix.patch" >> patches/series
  fi

  if ! grep 'setattr_prepare' fs/attr.c > /dev/null ; then
      cp -p ../../patches/qnap-fs_attr-fix.patch patches/platform/
      echo "platform/qnap-fs_attr-fix.patch" >> patches/series
  fi

  if ! grep 'usb_init_pool_max' include/linux/usb/hcd.h > /dev/null ; then
    if [ -e arch/arm/mach-comcerto/board-optimus.c ]; then
      cp -p ../../patches/google-hcd_h-fix.patch patches/platform/
      echo "platform/google-hcd_h-fix.patch" >> patches/series
    fi
  fi

  if [ -e arch/arm/mach-comcerto/board-optimus.c ]; then
      cp -p ../../patches/google-printk-fix.patch patches/platform/
      echo "platform/google-printk-fix.patch" >> patches/series
  fi

  #if grep EXT4_MAX_BLOCK_LOG_SIZE fs/ext4/ext4.h > /dev/null ; then
  #  if grep 'ext4_grpblk_t counters\[16\]' fs/ext4/mballoc.c > /dev/null ; then
  #    cp -p ../../patches/zyxel-nas5xx-ext4-mballoc-counters.patch patches/platform/
  #    echo "platform/zyxel-nas5xx-ext4-mballoc-counters.patch" >> patches/series
  #  fi
  #fi

  cp -p ../../patches/mspd-mm-enlarge-stack-guard-gap.patch patches/platform/
  sed -i 's|bugfix/all/mm-enlarge-stack-guard-gap.patch|platform/mspd-mm-enlarge-stack-guard-gap.patch|g' patches/series

  cp -p ../../patches/mspd-mm-larger-stack-guard-gap-between-vmas.patch patches/platform/
  sed -i 's|bugfix/all/mm-larger-stack-guard-gap-between-vmas.patch|platform/mspd-mm-larger-stack-guard-gap-between-vmas.patch|g' patches/series

  cp -p ../../patches/mspd-fs-fix-abi-change-for-aufs-f_setfl-fix.patch patches/platform/
  sed -i 's|debian/fs-fix-abi-change-for-aufs-f_setfl-fix.patch|platform/mspd-fs-fix-abi-change-for-aufs-f_setfl-fix.patch|g' patches/series

#linux-3.2.54_cgroup-update.patch

      cp -p ../../patches/linux-3.2_cgroup_add_xattr.patch patches/platform/
      echo "platform/linux-3.2_cgroup_add_xattr.patch" >> patches/series

      #cp -p ../../patches/linux-3.2_return_address.patch patches/platform/
      #echo "platform/linux-3.2_return_address.patch" >> patches/series

      cp -p ../../patches/linux-3.2_drivers-staging-rtl8712-ieee80211.patch patches/platform/
      echo "platform/linux-3.2_drivers-staging-rtl8712-ieee80211.patch" >> patches/series

      #cp -p ../../patches/linux-3.2_rc.patch patches/platform/
      #echo "platform/linux-3.2_rc.patch" >> patches/series

      cp -p ../../patches/linux-3.2_ubifs-no-date-time.patch patches/platform/
      echo "platform/linux-3.2_ubifs-no-date-time.patch" >> patches/series

      cp -p ../../patches/linux-3.2_af-packet.patch patches/platform/
      echo "platform/linux-3.2_af-packet.patch" >> patches/series

      cp -p ../../patches/linux-3.2_ath6kl-sdio.patch patches/platform/
      echo "platform/linux-3.2_ath6kl-sdio.patch" >> patches/series

      cp -p ../../patches/linux-3.2_drivers-staging-rtl8712-ieee80211-part2.patch patches/platform/
      echo "platform/linux-3.2_drivers-staging-rtl8712-ieee80211-part2.patch" >> patches/series

      cp -p ../../patches/linux-3.2_serial-8250.patch patches/platform/
      echo "platform/linux-3.2_serial-8250.patch" >> patches/series

      cp -p ../../patches/linux-3.2_ncpfs.patch patches/platform/
      echo "platform/linux-3.2_ncpfs.patch" >> patches/series

      cp -p ../../patches/linux-3.2_atm-iphase.patch patches/platform/
      echo "platform/linux-3.2_atm-iphase.patch" >> patches/series

      cp -p ../../patches/linux-3.2_dvb-frontends.patch patches/platform/
      echo "platform/linux-3.2_dvb-frontends.patch" >> patches/series

      #cp -p ../../patches/linux-3.2_cifs.patch patches/platform/
      #echo "platform/linux-3.2_cifs.patch" >> patches/series

      cp -p ../../patches/linux-3.2_blkdev.patch patches/platform/
      echo "platform/linux-3.2_blkdev.patch" >> patches/series

      #cp -p ../../patches/linux-3.2_libsas.patch patches/platform/
      #echo "platform/linux-3.2_libsas.patch" >> patches/series

  if ! grep txopt_get include/net/ipv6.h > /dev/null ; then
      cp -p ../../patches/linux-3.2_ipv6-txopt.patch patches/platform/
      echo "platform/linux-3.2_ipv6-txopt.patch" >> patches/series
  fi

      cp -p ../../patches/linux-3.2_nfs.patch patches/platform/
      echo "platform/linux-3.2_nfs.patch" >> patches/series

      cp -p ../../patches/linux-3.2_scsi_netlink.patch patches/platform/
      echo "platform/linux-3.2_scsi_netlink.patch" >> patches/series

      cp -p ../../patches/linux-3.2_mmc-ioctl.patch patches/platform/
      echo "platform/linux-3.2_mmc-ioctl.patch" >> patches/series

      cp -p ../../patches/linux-3.2_drm.patch patches/platform/
      echo "platform/linux-3.2_drm.patch" >> patches/series

  if ! grep 'struct sk_buff;' include/linux/kobject.h > /dev/null ; then
      cp -p ../../patches/linux-3.2_kobject-skbuff.patch patches/platform/
      echo "platform/linux-3.2_kobject-skbuff.patch" >> patches/series
  fi

      cp -p ../../patches/ebec18a6d3aa1e7d84aab16225e87fd25170ec2b.patch patches/platform/
      echo "platform/ebec18a6d3aa1e7d84aab16225e87fd25170ec2b.patch" >> patches/series

#linux-3.2.54_gcc6.patch

      cp -p ../../patches/linux-3.2_gcc6.patch patches/platform/
      echo "platform/linux-3.2_gcc6.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-ip6_tnl_4rd.patch patches/platform/
      echo "platform/zyxel-nas5xx-ip6_tnl_4rd.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-pfe-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-pfe-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-eeprom-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-eeprom-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-ocf-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-ocf-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-otg-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-otg-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-vwd-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-vwd-fixes.patch" >> patches/series

#      cp -p ../../patches/zyxel-nas5xx-wdt-fixes.patch patches/platform/
#      echo "platform/zyxel-nas5xx-wdt-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-xhci-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-xhci-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-xhci-c2k-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-xhci-c2k-fixes.patch" >> patches/series

  if ! grep cond_resched net/ipv4/udp.c > /dev/null ; then
      cp -p ../../patches/zyxel-nas5xx-udp-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-udp-fixes.patch" >> patches/series
  fi

  if grep '#if 0' drivers/tty/tty_ldisc.c > /dev/null ; then
      cp -p ../../patches/zyxel-nas5xx-tty-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-tty-fixes.patch" >> patches/series
  fi

      cp -p ../../patches/zyxel-nas5xx-mach-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-mach-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-mach-otp-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-mach-otp-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-mach-pcie-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-mach-pcie-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-mach-reset-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-mach-reset-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-m25p80-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-m25p80-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-net-core-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-net-core-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-netfilter-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-netfilter-fixes.patch" >> patches/series

      cp -p ../../patches/zyxel-nas5xx-md-dm-crypt-fixes.patch patches/platform/
      echo "platform/zyxel-nas5xx-md-dm-crypt-fixes.patch" >> patches/series

  if ! grep -E 'VM_MAX_READAHEAD.*512' include/linux/mm.h > /dev/null ; then
      cp -p ../../patches/zyxel-nas5xx-mm-increase-readahead.patch patches/platform/
      echo "platform/zyxel-nas5xx-mm-increase-readahead.patch" >> patches/series
  fi

  quilt push -a 2>&1 1>../${p}.patch.log
  r=$?

  if [ $r != 0 ]; then
    cat ../${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
    echo Failed $r
    exit $r
  fi

  echo "Copying linux-${lxdebkb}${lxsoceb} to debian..."
  mkdir ../linux-${lxdebkb}${lxsoceb}-debian
  cp -ap . ../linux-${lxdebkb}${lxsoceb}-debian/


  if grep 'arm-marvell-linux-gnueabi' Makefile > /dev/null ; then
    sed -i s/'CROSS_COMPILE.*?=.*'/'CROSS_COMPILE ?= $(CONFIG_CROSS_COMPILE:"%"=%)'/g Makefile
  fi

  if ! grep 'sound/ firmware/' Makefile > /dev/null ; then
    sed -i 's|drivers-y\t:= drivers/ sound/|drivers-y\t:= drivers/ sound/ firmware/|g' Makefile

    sed -i 's|-include $(srctree)/$(obj)/Makefile|include $(srctree)/$(obj)/Makefile|g' scripts/Makefile.fwinst
  fi

  if grep '.h $(srctree)/include/generated/' Makefile > /dev/null ; then
    sed -i 's|.h $(srctree)/include/generated/|.h include/generated/|g' Makefile
  fi

  if [ -e arch/arm/mach-comcerto/include/mach/ecc.h ]; then
    if grep -E '^#define ECC_UNCORR_ERR_HAMM.*0x4' arch/arm/mach-comcerto/include/mach/ecc.h > /dev/null ; then
      if grep -E '^#define ECC_UNCORR_ERR_HAMM.*(1 << 2)' arch/arm/mach-comcerto/include/mach/ecc.h > /dev/null ; then
        sed -i '/^#define ECC_UNCORR_ERR_HAMM.*0x4/d' arch/arm/mach-comcerto/include/mach/ecc.h
      fi
    fi
  fi

  if [ -e arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h -a -e arch/arm/mach-comcerto/clock.c ]; then
    #sed -i s/'define PLL'/'define C2K_CLK_PLL'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    #sed -i s/'define C2K_CLK_PLL_'/'define PLL_'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h

    sed -i s/'define PLL0\t'/'define C2K_CLK_PLL0\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL1\t'/'define C2K_CLK_PLL1\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL2\t'/'define C2K_CLK_PLL2\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL3\t'/'define C2K_CLK_PLL3\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h

    sed -i s/'case PLL0:'/'case C2K_CLK_PLL0:'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'case PLL1:'/'case C2K_CLK_PLL1:'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'case PLL2:'/'case C2K_CLK_PLL2:'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'case PLL3:'/'case C2K_CLK_PLL3:'/g arch/arm/mach-comcerto/clock.c

    sed -i s/' PLL0)'/' C2K_CLK_PLL0)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/' PLL1)'/' C2K_CLK_PLL1)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/' PLL2)'/' C2K_CLK_PLL2)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/' PLL3)'/' C2K_CLK_PLL3)'/g arch/arm/mach-comcerto/clock.c

    sed -i s/'(PLL0)'/'(C2K_CLK_PLL0)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'(PLL1)'/'(C2K_CLK_PLL1)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'(PLL2)'/'(C2K_CLK_PLL2)'/g arch/arm/mach-comcerto/clock.c
    sed -i s/'(PLL3)'/'(C2K_CLK_PLL3)'/g arch/arm/mach-comcerto/clock.c

    sed -i s/'define PLL_VSEL\t'/'define C2K_CLK_PLL_VSEL\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL_LOCK_EN\t'/'define C2K_CLK_PLL_LOCK_EN\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL_BYPASS\t'/'define C2K_CLK_PLL_BYPASS\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define PLL_RESET\t'/'define C2K_CLK_PLL_RESET\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h

    sed -i s/'define PLL_LOCK\t'/'define C2K_CLK_PLL_LOCK\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h

    sed -i s/'define SCPRE'/'define C2K_CLK_SCPRE'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define SCALL'/'define C2K_CLK_SCALL'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
    sed -i s/'define ISO_EN'/'define C2K_CLK_ISO_EN'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/clk-rst.h
  fi

  if grep 'int readFile(struct file \*fp,int \*buf,int readlen)' arch/arm/mach-comcerto/gpio.c > /dev/null ; then
    sed -i s/'readFile(struct file \*fp,int \*buf'/'readFile(struct file *fp,char *buf'/g arch/arm/mach-comcerto/gpio.c
  fi

  if grep 'int set_buzzer(unsigned long bz_data)' arch/arm/mach-comcerto/gpio.c > /dev/null ; then
    sed -i s/'int set_buzzer'/'void set_buzzer'/g arch/arm/mach-comcerto/gpio.c
  fi

  if grep 'static int gpio_ioctl' arch/arm/mach-comcerto/gpio.c > /dev/null ; then
    sed -i s/'static int gpio_ioctl'/'static long gpio_ioctl'/g arch/arm/mach-comcerto/gpio.c
  fi

  if grep 'define GPIO_' arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h > /dev/null ; then
    sed -i s/'define GPIO_0\t'/'define C2K_GPIO_0\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h
    sed -i s/'define GPIO_1\t'/'define C2K_GPIO_1\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h
    sed -i s/'define GPIO_2\t'/'define C2K_GPIO_2\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h
    sed -i s/'define GPIO_6\t'/'define C2K_GPIO_6\t'/g arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h

    for g in `seq 3 5` `seq 7 29`; do
      for f in arch/arm/mach-comcerto/include/mach/board-*.h ; do
        sed -i s/'GPIO_'$g/'C2K_GPIO_'$g/g $f
      done

      sed -i s/'GPIO_'$g/'C2K_GPIO_'$g/g arch/arm/mach-comcerto/include/mach/comcerto-2000/gpio.h
    done

    for f in arch/arm/mach-comcerto/include/mach/board-*.h ; do
      sed -i s/'GPIO_0 '/'C2K_GPIO_0 '/g $f
      sed -i s/' GPIO_0'/' C2K_GPIO_0'/g $f

      sed -i s/'GPIO_1 '/'C2K_GPIO_1 '/g $f
      sed -i s/' GPIO_1'/' C2K_GPIO_1'/g $f

      sed -i s/'GPIO_2 '/'C2K_GPIO_2 '/g $f
      sed -i s/' GPIO_2'/' C2K_GPIO_2'/g $f

      sed -i s/'GPIO_6'/'C2K_GPIO_6'/g $f
    done
  fi

  for n in UART SPI I2C ; do
    if grep 'define IRQ_'${n} arch/arm/mach-comcerto/include/mach/comcerto-2000/irqs.h > /dev/null ; then
      sed -i s/'define IRQ_'${n}/'define C2K_IRQ_'${n}/g arch/arm/mach-comcerto/include/mach/comcerto-2000/irqs.h

      for f in arch/arm/mach-comcerto/board-*.c arch/arm/mach-comcerto/comcerto-2000.c drivers/spi2/busses/comcerto_spi.c ; do
        sed -i s/' IRQ_'${n}/' C2K_IRQ_'${n}/g $f
      done
    fi
  done

  sed -i 's|-I ../../crypto/ocf|-I$(src)/../../crypto/ocf|g' drivers/epn1802-ocf/Makefile
  sed -i 's|-include $(src)/$(PFE_DIR)/c2000/version.h|-include $(PWD)/$(src)/$(PFE_DIR)/c2000/version.h|g' drivers/pfe_ctrl/Makefile
  sed -i 's|-T$(src)/control_link.lds|-T$(PWD)/$(src)/control_link.lds|g' drivers/pfe_ctrl/Makefile

  sed -i 's|cap.nr_ports for < AHCI 1.3 \*/|cap.nr_ports */|g' drivers/ata/libahci.c
  sed -i 's|(!port_map && vers < 0x10300)|(!port_map)|g' drivers/ata/libahci.c

  if ! grep 'mix_pool_bytes(struct entropy_store \*r, const void \*in, int bytes)' drivers/char/random.c > /dev/null ; then
    sed -i 's|mix_pool_bytes(\&input_pool, buf, wordcount\*4);|mix_pool_bytes(\&input_pool, buf, wordcount*4, NULL);|g' drivers/char/random.c
  fi

  if [ -e net/ipv6/output_core.c ]; then
    if ! grep output_core net/ipv6/Makefile > /dev/null ; then
      echo 'obj-$(CONFIG_INET) += output_core.o' >> net/ipv6/Makefile
    fi
  fi

  if grep -E '^header-y \+= xt_layer7.h' include/linux/netfilter/Kbuild > /dev/null ; then
    if [ ! -e include/linux/netfilter/xt_layer7.h ]; then
      #sed -i s/'^header-y += xt_layer7.h'/''/g include/linux/netfilter/Kbuild
      sed -i '/^header-y += xt_layer7.h/d' include/linux/netfilter/Kbuild
    fi
  fi

  cd ..

  rm -f a
  rm -f b
  ln -sf linux-${lxdebkb}${lxsoceb}-debian a
  ln -sf linux-${lxdebkb}${lxsoceb} b

  echo "Comparing linux-${lxdebkb}${lxsoceb}-debian to linux-${lxdebkb}${lxsoceb}..."
  lxdiff a b 084-mspd-kernel_linux_3_2_post.patch

  rm -f a
  rm -f b

  p=linux-${lxdebkb}${lxsoceb}-debian-post
  echo "Patching linux-${lxdebkb}${lxsoceb}-debian..."

  cd linux-${lxdebkb}${lxsoceb}/

  patch -p1 -R < ../084-mspd-kernel_linux_3_2_post.patch 2>&1 1>../${p}.patch.log

  cp -p ../084-mspd-kernel_linux_3_2_post.patch patches/platform/

  echo "platform/084-mspd-kernel_linux_3_2_post.patch" >> patches/series

  quilt push -a 2>&1 1>../${p}.patch.log
  r=$?

  if [ $r != 0 ]; then
    cat ../${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
    echo Failed $r
    exit $r
  fi

  sed -i s/'KBUILD_IMAGE := zImage'/'KBUILD_IMAGE := uImage'/g arch/arm/Makefile
  sed -i 's|boot/zImage|boot/uImage|g' debian/rules.real

  cd ..

  cd ..
fi


if [ ! -e linux ]; then
  echo "Copying linux-${lxdebkb}${lxsoceb}..."
  mkdir linux
  cp -ap combine/linux-${lxdebkb}${lxsoceb}/. linux/
  ./combine-config.sh
fi

echo OK
