#!/bin/sh
. ./combine-env.sh

lxsocta=linux-${lxsocka}
lxdebta=linux_${lxdebva}
lxdebtb=linux_${lxdebvb}

mkdir -p archives
cd archives

echo "Downloading debian linux_${lxdebkb}..."
wget -N ${lxdebub}/${lxdebtb}.debian.tar.xz
cd ..

echo "Extracting linux-${lxsocka}..."
tar xJf archives/${lxsocta}.tar.xz
mv linux-${lxsocka} linux

cd linux/

rm -f arch/arm/boot/compressed/lib1funcs.S
rm -f arch/arm/boot/compressed/piggy.gzip
rm -f arch/arm/boot/compressed/vmlinux
rm -f arch/arm/boot/compressed/vmlinux.lds
rm -f arch/arm/kernel/vmlinux.lds

tar xJf ../archives/${lxdebtb}.debian.tar.xz

echo "Patching linux-${lxsocka}..."
mv patches zy-patches
mv debian/patches .
mv zy-patches/generic patches/
mv zy-patches/platform patches/
mv patches/series patches/series-deb
cp -p zy-patches/series patches/

cp -p ../linux-${lxsocka}-to-${lxdeblb}*.patch patches/

if [ "${lxdebva}" != "${lxdebvb}" ]; then
  echo "linux-${lxsocka}-to-${lxdeblb}.patch" >> patches/series
fi
echo "linux-${lxsocka}-to-${lxdeblb}-extra.patch" >> patches/series
echo "linux-${lxsocka}-to-${lxdeblb}-force.patch" >> patches/series
echo "linux-${lxsocka}-to-${lxdeblb}-nohw.patch" >> patches/series

cat patches/series-deb >> patches/series

sed -i 's|debian/fs-fix-abi-change-for-aufs-f_setfl-fix.patch||g' patches/series

sed -i 's|debian/libata-avoid-abi-change-in-3.2.62.patch||g' patches/series
sed -i 's|debian/revert-libata-ignore-spurious-phy-event-on-lpm-polic.patch||g' patches/series

# mycloud only
if [ -e drivers/hwmon/wd-fan.c ]; then
  sed -i 's|debian/rtnetlink-avoid-ABI-change-in-3.2.34.patch||g' patches/series
  sed -i 's|debian/dm-avoid-ABI-change-in-3.2.41.patch||g' patches/series
fi

if false; then
sed -i 's|bugfix/.*/drm-.*||g' patches/series
sed -i 's|features/all/drm/.*||g' patches/series
sed -i 's|features/all/bql/tg3-.*||g' patches/series
sed -i 's|features/all/igb/.*||g' patches/series
sed -i 's|features/all/line6/.*||g' patches/series
sed -i 's|features/all/xen/.*||g' patches/series
sed -i 's|features/x86/hyperv/.*||g' patches/series

sed -i 's|bugfix/ia64/nouveau-ACPI-support-is-dependent-on-X86.patch||g' patches/series
sed -i 's|bugfix/x86/revert-perf-x86-amd-ibs-fix-waking-up-from-s3-for-amd-family-10h.patch||g' patches/series

sed -i 's|bugfix/all/firmware-remove-redundant-log-messages-from-drivers.patch||g' patches/series

sed -i 's|features/all/drivers-media-dvb-usb-af9005-request_firmware.patch||g' patches/series
sed -i 's|features/all/sound-pci-cs46xx-request_firmware.patch||g' patches/series

sed -i 's|features/all/fs-add-link-restrictions.patch||g' patches/series
sed -i 's|features/all/fs-add-link-restriction-audit-reporting.patch||g' patches/series
sed -i 's|features/all/fs-handle-failed-audit_log_start-properly.patch||g' patches/series
sed -i 's|features/all/fs-prevent-use-after-free-in-auditing-when-symlink-f.patch||g' patches/series

sed -i 's|features/all/debugfs-add-mode-uid-and-gid-options.patch||g' patches/series
sed -i 's|debian/debugfs-set-default-mode-to-700.patch||g' patches/series

sed -i 's|debian/.*ABI-change-in-3\.2\.55\.patch||g' patches/series
sed -i 's|debian/.*abi-change-in-3\.2\.57\.patch||g' patches/series
sed -i 's|debian/.*abi-change-in-3\.2\.59\.patch||g' patches/series

sed -i 's|debian/.*abi-change-in-3\.2\.6.\.patch||g' patches/series
sed -i 's|debian/.*abi-change-in-3\.2\.7.\.patch||g' patches/series
sed -i 's|debian/.*abi-change-in-3\.2\.8.\.patch||g' patches/series

sed -i 's|debian/revert-alsa-enable-config_zone_dma.patch||g' patches/series
sed -i 's|debian/alsa-avoid-abi-change-for-cve-2014-4652-fix.patch||g' patches/series
sed -i 's|debian/drivers-net-avoid-abi-change-for-ufo-ipv6-fix.patch||g' patches/series
sed -i 's|debian/revert-acpica-utilities-split-io-address-types-from-.patch||g' patches/series
sed -i 's|debian/revert-libata-ignore-spurious-phy-event-on-lpm-polic.patch||g' patches/series
sed -i 's|debian/af_unix-avoid-abi-changes.patch||g' patches/series
sed -i 's|debian/enclosure-fix-abi-change-in-2.6.32.70.patch||g' patches/series
sed -i 's|debian/fix-abi-changes-for-cve-2013-4312-fix.patch||g' patches/series
sed -i 's|debian/revert-libata-align-ata_device-s-id-on-a-cacheline.patch||g' patches/series
sed -i 's|debian/revert-net-ipv6-add-sysctl-option-accept_ra_min_hop_limit.patch||g' patches/series
sed -i 's|debian/fs-move-procfs-ecryptfs-stacking-check-into-ecryptfs.patch||g' patches/series
fi

quilt push -a


if [ -e drivers/pfe_ctrl/Makefile ]; then
  if [ ! -e drivers/pfe_ctrl/Kconfig ]; then
    cat <<EOFPFECFG | tee drivers/pfe_ctrl/Kconfig > /dev/null
#
# MindSpeed pfe support (added by Andy, 2014/07/08)
#

config PFE_CTRL
	tristate "MindSpeed pfe support"
	default m
	---help---
		MindSpeed pfe kernel module
		(copied from "MindSpeed_OpenWRT/sdk-comcerto-openwrt-c2k_nas_3.1-rc2/package/mindspeed/pfe_ctrl")
EOFPFECFG
  fi

  if ! grep pfe_ctrl drivers/Kconfig >/dev/null ; then
    sed -i 's|source "drivers/devfreq/Kconfig"|source "drivers/devfreq/Kconfig"\n\nsource "drivers/pfe_ctrl/Kconfig"|g' drivers/Kconfig
  fi

  if ! grep pfe_ctrl drivers/Makefile >/dev/null ; then
    echo 'obj-$(CONFIG_PFE_CTRL)		+= pfe_ctrl/' >> drivers/Makefile
  fi
fi

sed -i 's|-I ../../crypto/ocf|-I$(src)/../../crypto/ocf|g' drivers/epn1802-ocf/Makefile
sed -i 's|-include $(src)/$(PFE_DIR)/c2000/version.h|-include $(PWD)/$(src)/$(PFE_DIR)/c2000/version.h|g' drivers/pfe_ctrl/Makefile
sed -i 's|-T$(src)/control_link.lds|-T$(PWD)/$(src)/control_link.lds|g' drivers/pfe_ctrl/Makefile


sed -i s/'KBUILD_IMAGE := zImage'/'KBUILD_IMAGE := uImage'/g arch/arm/Makefile
sed -i 's|boot/zImage|boot/uImage|g' debian/rules.real

cd ..

./combine-config.sh

#EOF

