#!/bin/sh
. ./combine-env.sh

# functions

ignore() {
  for f in `cat $1` ; do
    of=$2/$f
    nf=$2-ignore/$f
    od=`dirname $of`
    nd=`dirname $nf`
    if [ -e $of ]; then
      mkdir -p $nd
      mv $of $nd
    fi
  done
}

restore() {
  for f in `cat $1` ; do
    of=$2/$f
    nf=$2-ignore/$f
    od=`dirname $of`
    nd=`dirname $nf`
    if [ -e $nf ]; then
      mkdir -p $od
      mv $nf $od
    fi
  done
}

extradiff() {
  rm -f c
  rm -f d
  ln -s $2 c
  ln -s $3 d
  for f in `cat $1` ; do
    # "x${f:0:7}" != "xfs/ext4"
    # "x${f}" != "xfs/ext4/super.c"
    # "x${f}" != "xdrivers/md/md.c"
    if [ "x${f}" != "xarch/arm/include/asm/pgtable.h" -a "x${f}" != "xdrivers/char/random.c" -a "x${f}" != "xinclude/linux/random.h" \
         -a "x${f}" != "xdrivers/usb/core/hub.c" -a "x${f}" != "xdrivers/usb/host/xhci-hub.c" -a "x${f}" != "xdrivers/usb/host/xhci-mem.c" -a "x${f}" != "xinit/main.c" -a "x${f}" != "xkernel/irq/handle.c" \
         -a "x${f}" != "xinclude/linux/if_vlan.h" -a "x${f}" != "xinclude/net/rtnetlink.h" -a "x${f}" != "xnet/8021q/vlan_core.c" -a "x${f}" != "xnet/core/dev.c" -a "x${f}" != "xnet/core/rtnetlink.c" \
         -a "x${f}" != "xblock/blk-settings.c" -a "x${f:0:11}" != "xdrivers/ata" -a "x${f}" != "xfs/ext4/file.c" \
         -a "x${f}" != "xdrivers/md/dm-crypt.c" -a "x${f}" != "xdrivers/md/raid5.c" -a "x${f}" != "xnet/key/af_key.c" \
         -a "x${f}" != "xkernel/softirq.c" -a "x${f:0:15}" != "xdrivers/mtd/ubi" \
         -a "x${f}" != "xdrivers/usb/host/xhci.c" -a "x${f}" != "xdrivers/usb/host/xhci.h" \
         -a "x${f}" != "xdrivers/usb/host/xhci-pci.c" -a "x${f}" != "xdrivers/usb/host/xhci-ring.c" ]; then
      diff -urN --exclude='*.orig' c/$f d/$f >> linux-${lxsocka}-to-${lxdeblb}-extra.patch
    fi
  done
  rm -f c
  rm -f d
}

extrapatch() {
  n=$1
  p=$2
  cd lt/linux-${lxsocka}
  echo "Patching ${n}..."
  patch -p1 < ../../${p}.patch 2>&1 1>../${p}.patch.log
  r=$?
  if [ $r != 0 ]; then
    cat ../${p}.patch.log | grep 'saving rejects to file' | cut -d - -f 2- | cut -d ' ' -f 6 | sed s/'.rej'/''/g
    echo ...failed
    exit $r
  fi
  cd ../..
}

# main

mkdir -p archives
cd archives
echo "Downloading debian linux_${lxdebka}..."
wget -N "${lxdebua}/linux_${lxdebka}.orig.tar.xz"
if [ ! -e linux_${lxdebka}.orig.tar.xz ]; then
  wget "${lxdebua}/linux-${lxdebka}.tar.xz" -O linux_${lxdebka}.orig.tar.xz
fi
wget -N "${lxdebua}/linux_${lxdebva}.debian.tar.xz"
wget -N "${lxdebua}/linux_${lxdebva}.dsc"

echo "Downloading debian linux_${lxdebkb}..."
wget -N "${lxdebub}/linux_${lxdebkb}.orig.tar.xz"
wget -N "${lxdebub}/linux_${lxdebvb}.debian.tar.xz"
wget -N "${lxdebub}/linux_${lxdebvb}.dsc"
cd ..

mkdir -p orig
cd orig

if [ ! -e linux-${lxdebka} -o -e linux-${lxdebka}-ignore ]; then
  echo "Extracting debian linux_${lxdebka}..."
  rm -rf linux-${lxdebka}/
  rm -rf linux-${lxdebka}-ignore/
  tar xJf ../archives/linux_${lxdebka}.orig.tar.xz
fi

if [ ! -e linux-${lxdebkb} -o -e linux-${lxdebkb}-ignore ]; then
  echo "Extracting debian linux_${lxdebkb}..."
  rm -rf linux-${lxdebkb}/
  rm -rf linux-${lxdebkb}-ignore/
  tar xJf ../archives/linux_${lxdebkb}.orig.tar.xz
fi

cd ..


if false; then
mkdir -p debian
cd debian

if [ ! -e linux-${lxdebka} -o -e linux-${lxdebka}-ignore ]; then
  echo "Extracting debian linux_${lxdebva}..."
  rm -rf linux-${lxdebka}/
  rm -rf linux-${lxdebka}-ignore/
  tar xJf ../archives/linux_${lxdebka}.orig.tar.xz
  cd linux-${lxdebka}/
  tar xJf ../../archives/linux_${lxdebva}.debian.tar.xz
  mv debian/patches .
  echo "Patching linux_${lxdebva}..."
  quilt push -a 2>&1 1>../linux_${lxdebva}-quilt.log
  cd ..
fi

if [ ! -e linux-${lxdebkb} -o -e linux-${lxdebkb}-ignore ]; then
  echo "Extracting debian linux_${lxdebvb}..."
  rm -rf linux-${lxdebkb}/
  rm -rf linux-${lxdebkb}-ignore/
  tar xJf ../archives/linux_${lxdebkb}.orig.tar.xz
  cd linux-${lxdebkb}/
  tar xJf ../../archives/linux_${lxdebvb}.debian.tar.xz
  mv debian/patches .
  echo "Patching linux_${lxdebvb}..."
  quilt push -a 2>&1 1>../linux_${lxdebvb}-quilt.log
  cd ..
fi

cd ..
fi


mkdir -p archives
cd archives
echo "Downloading linux-${lxsocka}..."
wget -N "${lxsocua}/linux-${lxsocka}.tar.xz"
cd ..

mkdir -p lt
cd lt

echo "Extracting linux-${lxsocka}..."
rm -rf linux-${lxsocka}/
rm -rf linux-${lxsocka}-ignore/
tar xJf ../archives/linux-${lxsocka}.tar.xz
cd ..

if false; then
ignore lists/combine-exclude-all.txt debian/linux-${lxdebka}
ignore lists/combine-exclude-all.txt debian/linux-${lxdebkb}

echo "Comparing debian linux-${lxdebva} linux-${lxdebvb}..."
rm -f a
rm -f b
ln -s debian/linux-${lxdebka} a
ln -s debian/linux-${lxdebkb} b
diff -urN --exclude='*.orig' a b > linux-${lxdebva}-to-${lxdebvb}.patch
rm -f a
rm -f b

restore lists/combine-exclude-all.txt debian/linux-${lxdebka}
restore lists/combine-exclude-all.txt debian/linux-${lxdebkb}
fi

ignore lists/combine-exclude-all.txt orig/linux-${lxdebka}
ignore lists/combine-exclude-all.txt orig/linux-${lxdebkb}
ignore lists/combine-exclude.txt orig/linux-${lxdebka}
ignore lists/combine-exclude.txt orig/linux-${lxdebkb}

if [ "${lxdebva}" != "${lxdebvb}" ]; then
  echo "Comparing debian linux-${lxdebka} linux-${lxdebkb}..."
  rm -f a
  rm -f b
  ln -s orig/linux-${lxdebka} a
  ln -s orig/linux-${lxdebkb} b
  diff -urN a b > linux-${lxdebka}-to-${lxdeblb}.patch
  rm -f a
  rm -f b

  extrapatch linux-${lxsocka} linux-${lxdebka}-to-${lxdeblb}
fi

rm -f linux-${lxsocka}-to-${lxdeblb}-extra.patch

echo "Comparing linux-${lxsocka}-to-${lxdeblb} linux-${lxdebkb}..."
extradiff lists/combine-exclude.txt lt/linux-${lxsocka} orig/linux-${lxdebkb}-ignore

extrapatch linux-${lxsocka}-to-${lxdeblb} linux-${lxsocka}-to-${lxdeblb}-extra

restore lists/combine-exclude.txt orig/linux-${lxdebka}
restore lists/combine-exclude.txt orig/linux-${lxdebkb}
restore lists/combine-exclude-all.txt orig/linux-${lxdebka}
restore lists/combine-exclude-all.txt orig/linux-${lxdebkb}

cd orig
rm -rf linux-${lxdebka}-ignore/
rm -rf linux-${lxdebkb}-ignore/
cd ..


echo "Comparing linux-${lxsocka}-to-${lxdeblb} linux-${lxdebkb}..."
mv linux-${lxsocka}-to-${lxdeblb}-extra.patch linux-${lxsocka}-to-${lxdeblb}-extra1.patch
extradiff lists/combine-force.txt lt/linux-${lxsocka} orig/linux-${lxdebkb}
mv linux-${lxsocka}-to-${lxdeblb}-extra.patch linux-${lxsocka}-to-${lxdeblb}-force.patch
mv linux-${lxsocka}-to-${lxdeblb}-extra1.patch linux-${lxsocka}-to-${lxdeblb}-extra.patch

extrapatch linux-${lxsocka}-to-${lxdeblb}-extra linux-${lxsocka}-to-${lxdeblb}-force


echo "Comparing linux-${lxsocka}-to-${lxdeblb} linux-${lxdebkb}..."
mv linux-${lxsocka}-to-${lxdeblb}-extra.patch linux-${lxsocka}-to-${lxdeblb}-extra1.patch
extradiff lists/combine-exclude-c2k-nohw.txt lt/linux-${lxsocka} orig/linux-${lxdebkb}
mv linux-${lxsocka}-to-${lxdeblb}-extra.patch linux-${lxsocka}-to-${lxdeblb}-nohw.patch
mv linux-${lxsocka}-to-${lxdeblb}-extra1.patch linux-${lxsocka}-to-${lxdeblb}-extra.patch

extrapatch linux-${lxsocka}-to-${lxdeblb}-extra linux-${lxsocka}-to-${lxdeblb}-nohw


ignore lists/combine-exclude-all.txt lt/linux-${lxsocka} 
ignore lists/combine-exclude-c2k.txt lt/linux-${lxsocka}
ignore lists/combine-exclude-c2k-extra.txt lt/linux-${lxsocka}

ignore lists/combine-exclude-all.txt orig/linux-${lxdebkb}

echo "Comparing linux-${lxdebkb} linux-${lxsocka}-to-${lxdeblb}-extra..."
rm -f e
rm -f f
ln -s orig/linux-${lxdebkb} e
ln -s lt/linux-${lxsocka}   f
diff -urN --exclude=powerpc --exclude='*.orig' e f > linux-${lxdebkb}-mspd-c2k.patch
rm -f e
rm -f f

restore lists/combine-exclude-all.txt orig/linux-${lxdebkb}

cd orig
rm -rf linux-${lxdebkb}-ignore/
cd ..

restore lists/combine-exclude-all.txt lt/linux-${lxsocka}

cd lt/linux-${lxsocka}-ignore/
grep -r 'config ' . | grep Kconfig | cut -d ":" -f2 > ../linux-${lxsocka}-mspd-c2k-configs.txt
cd ../..

#EOF
